// Extension from mousetrap-record
interface MousetrapStatic {
	record(callback: (this: MousetrapStatic, sequence: string[]) => void): void;
}

// d3-dtree

declare namespace d3dTree {
	interface Person {
		name: string;
	}
	interface Marriage {
		spouse: Person;
		/**
		 * List of children nodes
		 */
		children: Person[];
	}

	interface DataItem {
		/**
		 * The name of the node
		 */
		name: string;
		/**
		 * The CSS class of the node
		 */
		class: string;
		/**
		 * The CSS class of the text in the node
		 */
		textClass: string;
		/**
		 * Generational height offset
		 */
		depthOffset?: number;
		/** Marriages is a list of nodes
		 * Each marriage has one spouse
		 */
		marriages: Marriage[];
		/**
		 * Custom data passed to renderers
		 */
		extra?: object;
	}

	interface Options {
		target: string;
		debug: boolean;
		width: number;
		height: number;
		hideMarriageNodes: boolean;
		marriageNodeSize: number;
		/**
		 *  Callbacks should only be overwritten on a need to basis.
		 * See the section about callbacks below.
		 */
		callbacks: {
		},
		margin: {
			top: number;
			right: number;
			bottom: number;
			left: number;
		},
		nodeWidth: number;
		styles: {
			node: string;
			linage: string;
			marriage: string;
			text: string;
		}
	}

	interface Tree {
		/**
		 * Reset zoom and position to initial state
		 * @param duration Default is 500
		 */
		resetZoom(duration?: number): void;
		/**
		 * Zoom to a specific position
		 * @param x
		 * @param y
		 * @param zoom = 1
		 * @param duration = 500
		 */
		zoomTo(x: number, y: number, zoom?: number, duration?: number): void;
		/**
		 * Zoom to a specific node
		 * @param nodeId
		 * @param zoom = 2
		 * @param duration = 500
		 */
		zoomToNode(nodeId: string, zoom?: number, duration?: number): void;
		/**
		 * Zoom to fit the entire tree into the viewport
		 * @param duration = 500
		 */
		zoomToFit(duration?: number): void;
	}

	interface dTree {
		init(data: DataItem[], options: Partial<Options>): Tree;
	}
}

declare const dTree: d3dTree.dTree;
