/**
 * Details week-to-week changes in children in the Nursery
 * @returns {DocumentFragment}
 */
App.Facilities.Nursery.childrenReport = function childrenReport() {
	const
		frag = new DocumentFragment(),

		Matron = S.Matron,
		nannies = App.Utils.sortedEmployees(App.Entity.facilities.nursery),
		NL = App.Entity.facilities.nursery.employeesIDs().size,
		CL = V.cribs.length,

		medianNannyIntelligence = NL ? findMedianNannyIntelligence() : null,
		medianNannyIntelligenceImplant = NL ? findMedianNannyIntelligenceImplant() : null;

	for (const child of V.cribs) {
		const childDiv = App.UI.DOM.appendNewElement("div", frag, '', "child-section");

		childDiv.append(childGrowTime(child));

		if (child.actualAge >= 3) {
			if (Matron) {
				if (Matron.fetish !== "none") {
					childDiv.append(matronFetishEffects(child));
				}

				childDiv.append(matronEducationEffects(child));
				// childDiv.append(matronFitnessEffects(child));
			}

			if (NL > 0) {
				const
					randomNanny = NL > 1 ? jsRandom(0, nannies.length - 1) : 0,
					nanny = nannies[randomNanny];

				if (nanny.fetish !== "none") {
					childDiv.append(nannyFetishEffects(child, nanny));
				}

				childDiv.append(nannyEducationEffects(child));
				// childDiv.append(nannyFitnessEffects(child));
			}

			if (multipleChildrenOverTargetAge(V.cribs.findIndex(c => c.ID === child.ID))) {
				childDiv.append(childFriendshipRivalries(child));
			}

			// TODO: rework these entirely
			if (Matron || nannies) {
				childDiv.appendChild(weightRulesEffects(child));
				childDiv.appendChild(musclesRulesEffects(child));
			}
		} else {
			// TODO:
		}
	}



	// MARK: Matron Effects

	function matronFetishEffects(child) {
		const
			chance = jsRandom(1, 100);

		if ((chance > 90 && child.fetish === "none") || chance > 95) {
			child.fetish = Matron.fetish;

			return `${child.slaveName} has taken a few cues from ${Matron.slaveName}, and ${newChildFetish(child.fetish)}. `;
		}
	}

	function matronEducationEffects(child) {
		// TODO: expand this
		const
			{ he, him, his } = getPronouns(Matron),

			theChildren = CL > 1 ? `the children` : `${child.slaveName}`;

		if (Matron.intelligence + Matron.intelligenceImplant > 65) {
			child.intelligenceImplant += 3;

			return `${Matron.slaveName} is so intelligent and well-educated that ${he} is able to teach ${theChildren} very effectively, and so ${CL > 1 ? `they gradually grow` : `${child.slaveName} gradually grows`} smarter. `;
		} else if (Matron.intelligenceImplant > 20) {
			child.intelligenceImplant += 2;

			return `${Matron.slaveName}'s education makes up for the fact that ${he} isn't the brightest and allows ${him} to teach ${theChildren} quite effectively, and so ${CL > 1 ? `they grow` : `${child.slaveName} grows`} a bit smarter. `;
		} else if (Matron.intelligence > 50) {
			child.intelligenceImplant += 2;

			return `Though ${Matron.slaveName} has had little to no formal education, ${his} natural brilliance allows ${him} to teach ${theChildren} quite effectively, and so ${CL > 1 ? `they grow` : `${child.slaveName} grows`} a bit smarter. `;
		} else {
			const
				totalSpan = App.UI.DOM.makeElement("span", `${Matron.slaveName} isn't the brightest and not well educated, `),
				damageSpan = App.UI.DOM.makeElement("span", `damaging the amount of real education ${theChildren} receive. `, "red");

			child.intelligenceImplant--;

			totalSpan.appendChild(damageSpan);

			return totalSpan;
		}
	}

	// function matronFitnessEffects(child) {
	// 	// TODO:
	// 	return;
	// }



	// MARK: Nanny Effects

	function nannyFetishEffects(child, slave) {
		const
			{ he } = getPronouns(child),
			chance = jsRandom(1, 100);

		if (chance > 85) {
			if (child.fetish === "none") {
				child.fetish = slave.fetish;

				return `${slave.slaveName} has left quite an impression on ${child.slaveName}, and ${he} ${newChildFetish(child.fetish)}. `;
			} else {
				if (chance > 90) {
					child.fetish = slave.fetish;

					return `${child.slaveName} seems to have taken to ${slave.slaveName}'s example, and ${newChildFetish(child.fetish)}. `;
				}
			}
		}
	}

	function nannyEducationEffects(child) {
		// TODO: redo this entire section
		// TODO: expand this
		const
			firstNanny = nannies[0],
			theNanniesAre = NL > 1 ? `The nannies are mostly` : `${firstNanny.slaveName} is`,
			theChildren = CL > 1 ? `the children` : child.slaveName;

		if (medianNannyIntelligence + medianNannyIntelligenceImplant > 65) {
			child.intelligenceImplant += 3;

			return `${theNanniesAre} very intelligent and well educated and are able to teach ${theChildren} very effectively. `;
		} else if (medianNannyIntelligence > 50) {
			child.intelligenceImplant += 2;

			return `${theNanniesAre} very intelligent and able to teach ${theChildren} quite effectively. `;
		} else if (medianNannyIntelligenceImplant > 25) {
			child.intelligenceImplant += 2;

			return `${theNanniesAre} very well educated and able to teach ${theChildren} quite effectively. `;
		} else if (medianNannyIntelligenceImplant > 15) {
			child.intelligenceImplant++;

			return `${theNanniesAre} well educated and able to teach ${theChildren} fairly effectively. `;
		}
	}

	// function nannyFitnessEffects(child) {
	// 	// TODO:
	// 	return;
	// }



	// MARK: Nursery Rules Effects

	function weightRulesEffects(child) {
		// TODO: redo this entire section
		// TODO: double check these classes, make sure they make sense
		const
			span = document.createElement("span"),
			{ he, He, His } = getPronouns(child);

		if (V.nurseryWeight) {
			const
				firstNanny = NL > 0 ? nannies[0] : null,
				caretaker = Matron ? Matron.slaveName : NL > 1 ? `A nanny` : firstNanny.slaveName;

			if (V.nurseryWeightSetting === 1) {
				const weightSpan = App.UI.DOM.makeElement("span", 'rapid weight gain.', ["health", "dec"]);

				if (child.weight < 200) {
					child.weight += 5;
				}

				span.append(`${He} is being fed an excessive amount of food, causing`, weightSpan);
			} else if (V.nurseryWeightSetting === 2) {
				const weightSpan = App.UI.DOM.makeElement("span", `decreases the amount of food ${he} eats. `, "improvement");

				if (child.weight > 10) {
					child.weight--;

					span.append(`${caretaker} notices ${he} is overweight and `, weightSpan);
				} else if (child.weight <= -10) {
					const weightSpan = App.UI.DOM.makeElement("span", `increases the amount of food ${he} eats. `, "improvement");

					child.weight++;

					span.append(`${caretaker} notices ${he} is underweight and `, weightSpan);
				} else {
					const weightSpan = App.UI.DOM.makeElement("span", 'currently a healthy weight;', ["change", "positive"]);

					span.append(`${He} is `, weightSpan, ` efforts will be made to maintain it. `);
				}
			} else if (V.nurseryWeightSetting === 0) {
				if (child.weight > -20) {
					const weightSpan = App.UI.DOM.makeElement("span", 'quickly sheds its gained weight.', ["health", "dec"]);

					child.weight -= 40;

					span.append(`${His} developing body `, weightSpan);
				}
			}
		} else {
			if (child.weight > -20) {
				const weightSpan = App.UI.DOM.makeElement("span", 'quickly sheds its gained weight. ', ["health", "dec"]);

				child.weight -= 40;

				span.append(`${His} developing body `, weightSpan);
			}
		}

		return span;
	}

	function musclesRulesEffects(child) {
		const div = document.createElement("div");

		// TODO: rewrite these
		// FIXME: this entire section needs a rewrite - numbers and text don't line up at all
		if (V.nurseryMuscles) {
			const
				firstNanny = NL > 0 ? nannies[0] : null,
				caretaker = Matron ? Matron.slaveName : NL > 1 ? `A nanny` : firstNanny.slaveName,
				{ His, He, he } = getPronouns(child);


			const muscleSpan = App.UI.DOM.makeElement("div", 'rapid muscle development.', "improvement");

			div.append(`${He} is being worked out as often as possible, resulting in `, muscleSpan);

			if (V.nurseryMusclesSetting === 2) {
				const muscleSpan = App.UI.DOM.makeElement("span", `decreases the amount of exercise ${he} receives. `, "improvement");

				if (child.muscles > 100) {
					child.muscles -= 5;
				}

				div.append(`${caretaker} notices ${he} is overly muscular and `, muscleSpan);
			} else if (V.nurseryMusclesSetting === 1) {
				if (child.muscles < -10) {
					const muscleSpan = App.UI.DOM.makeElement("span", `increases the amount of exercise ${he} receives. `, "improvement");

					child.muscles--;

					div.append(`${caretaker} notices ${he} is weak and `, muscleSpan);
				} else if (child.muscles > 10) {
					const muscleSpan = App.UI.DOM.makeElement("span", 'a healthy musculature;', ["change", "positive"]);

					child.muscles++;

					div.append(`${He} has `, muscleSpan, ` efforts will be made to maintain it. `);
				} else {
					const muscleSpan = App.UI.DOM.makeElement("span", 'quickly loses its gained muscle.', ["health", "dec"]);

					div.append(`${His} developing body `, muscleSpan);
				}
			} else if (V.nurseryMusclesSetting === 0) {
				const muscleSpan = App.UI.DOM.makeElement("span", 'quickly loses its gained muscle.', ["health", "dec"]);

				if (child.muscles > 100) {
					child.muscles -= 40;
					div.append(`${His} developing body `, muscleSpan);
				}
			}
		}

		return div;
	}



	// MARK: Miscellaneous Functions

	function childFriendshipRivalries(child) {
		const cribsCopy = Array.from(V.cribs);

		cribsCopy.splice(V.cribs.findIndex(c => c.ID === child.ID));

		for (const target of cribsCopy) {
			const
				becomeFriends = () => `${child.slaveName} and ${target.slaveName} have realized that they have more in common that they originally thought, and have become friends. `,
				becomeRivals = () => `${child.slaveName} and ${target.slaveName} have more differences between them than they could put aside and have become rivals. `,
				haveSameFetish = () => child.fetish === target.fetish && child.fetish !== "none",
				haveSameBehavioralQuirk = () => child.behavioralQuirk && child.behavioralQuirk === target.behavioralQuirk && child.behavioralQuirk !== "none",
				haveSameSexualQuirk = () => child.sexualQuirk && child.sexualQuirk === target.sexualQuirk && child.sexualQuirk !== "none",

				div = document.createElement("div"),

				{ his } = getPronouns(target),
				chance = jsRandom(1, 100);

			let
				friend = 0,
				rival = 0;

			if (target.actualAge >= 3) {
				if (haveSameFetish()) {
					div.append(`${sameFetish(child, target)}, a fact over which they bond. `);

					friend++;
				}

				if (haveSameBehavioralQuirk()) {	// TODO:
					div.append(`Since ${sameBehavioralQuirk(child, target)}, they learn to get along a bit better. `);

					friend++;
				}

				if (haveSameSexualQuirk()) {
					div.append(`Because ${sameSexualQuirk(child, target, haveSameBehavioralQuirk())}, the two grow a bit closer. `);

					friend++;
				}

				if (target.fetish === "sadist" || target.fetish === "dom") {
					div.append(`${target.slaveName} is a ${target.fetish}, and ${child.slaveName} is often ${his} target, which ${child.slaveName} doesn't particularly like. `);

					rival++;
				} else if (child.fetish === "sadist" || child.fetish === "dom") {
					div.append(`${child.slaveName} is a ${child.fetish}, and ${target.slaveName} is often ${his} target, which ${target.slaveName} doesn't particularly like. `);

					rival++;
				}

				if (areRelated(child, target) || areCousins(child, target)) {
					if (areRelated(child, target)) {
						if (rival) {
							div.append(`${child.slaveName} and ${target.slaveName} are siblings, and find it difficult to really stay mad at each other, and they make up their differences somewhat. `);

							friend += 2;
						} else {
							div.append(`${child.slaveName} and ${target.slaveName} are siblings, a fact that draws them closer together. `);

							friend += 2;
						}
					} else {
						if (rival) {
							div.append(`${child.slaveName} and ${target.slaveName} are cousins, and find it difficult to really stay mad at each other, and they make up their differences somewhat. `);

							friend++;
						} else {
							div.append(`${child.slaveName} and ${target.slaveName} are cousins, a fact that draws them closer together. `);

							friend++;
						}
					}
				}

				if (friend) {
					if (rival) {
						if (friend > rival && child.relationshipTarget !== target.ID) {
							if (chance > 75) {
								div.append(becomeFriends());

								child.relationship = 1;
								child.relationshipTarget = target.ID;

								target.relationship = 1;
								target.relationshipTarget = child.ID;
							}
						}
					} else {
						if (chance > 60) {
							div.append(becomeRivals());

							child.relationship = 1;
							child.relationshipTarget = target.ID;

							target.relationship = 1;
							target.relationshipTarget = child.ID;
						}
					}
				}

				if (rival) {
					if (friend) {
						if (rival > friend) {
							if (chance > 75) {
								div.append(becomeRivals());
							}
						}
					} else {
						if (chance > 60) {
							div.append(becomeFriends());
						}
					}
				}
			}

			return div;
		}

		function sameFetish(child, target) {
			switch (child.fetish) {
				case "submissive":
					return `${child.slaveName} and ${target.slaveName} are both sexually submissive`;
				case "cumslut":
					return `Neither ${child.slaveName} nor ${target.slaveName} can get enough cum`;
				case "humiliation":
					return `Both ${child.slaveName} and ${target.slaveName} have a fetish for humiliation`;
				case "buttslut":
					return `Neither ${child.slaveName} nor ${target.slaveName} can get enough assplay`;
				case "boobs":
					return `${child.slaveName} and ${target.slaveName} both love breastplay`;
				case "sadist":
					return `Both ${child.slaveName} are ${target.slaveName} are sadists`;
				case "masochist":
					return `${child.slaveName} and ${target.slaveName} are both masochists`;
				case "dom":
					return `Both ${child.slaveName} and ${target.slaveName} are sexually dominant`;
				case "pregnancy":
					return `The idea of pregnancy titillates both ${child.slaveName} and ${target.slaveName}`;
				default:
					throw `Unexpected value ${child.fetish} in sameFetish(). Please report this.`;
			}
		}

		function sameBehavioralQuirk(child, target) {
			switch (child.behavioralQuirk) {
				case "confident":
					return `${child.slaveName} and ${target.slaveName} are both naturally confident`;
				case "cutting":
					return `both ${child.slaveName} and ${target.slaveName} have a cutting wit about them`;
				case "funny":
					return `${child.slaveName} and ${target.slaveName} can both make the other laugh`;
				case "fitness":
					return `${child.slaveName} and ${target.slaveName} both love to try to stay in shape`;
				case "adores women":
					return `${child.slaveName} and ${target.slaveName} both adore women`;
				case "adores men":
					return `both ${child.slaveName} are ${target.slaveName} adore men`;
				case "insecure":
					return `${child.slaveName} and ${target.slaveName} are both equally insecure`;
				case "sinful":
					return `both ${child.slaveName} and ${target.slaveName} love breaking cultural and religious mores`;
				case "advocate":
					return `${child.slaveName} and ${target.slaveName} can both make a strong case for slavery`;
				default:
					throw `Unexpected value ${child.behavioralQuirk} in sameBehavioralQuirk(). Please report this.`;
			}
		}

		// TODO: incorporate minimumSlaveAge
		function sameSexualQuirk(child, target, sameQuirk = false) {
			switch (child.sexualQuirk) {
				case "gagfuck queen":
					return `${sameQuirk ? `the two also` : `${child.slaveName} and ${target.slaveName} both`} love having their little throats fucked`;
				case "painal queen":
					return `${sameQuirk ? `neither` : `neither ${child.slaveName} nor ${target.slaveName}`} can get enough painal`;
				case "strugglefuck queen":
					return `${sameQuirk ? `the two also` : `${child.slaveName} and ${target.slaveName} both`} love to put up a struggle during sex`;
				case "tease":
					return `${sameQuirk ? `the two are also both teases` : `${child.slaveName} and ${target.slaveName} are both teases`}`;
				case "romantic":
					return `${sameQuirk ? `the two also` : `${child.slaveName} and ${target.slaveName} both`} see the world from under a romantic lens`;
				case "perverted":
					return `${sameQuirk ? `the two are also` : `${child.slaveName} and ${target.slaveName} are both`} big-time perverts`;
				case "caring":
					return `${sameQuirk ? `the two also` : `${child.slaveName} and ${target.slaveName} both`} care about their partners`;
				case "unflinching":
					return `${sameQuirk ? `the two can also` : `${child.slaveName} and ${target.slaveName} can both`} take whatever their partner might throw at them `;
				case "size queen":
					return `${sameQuirk ? `the two also` : `${child.slaveName} and ${target.slaveName} both`} have a love for huge cock`;
				default:
					throw `Unexpected value ${child.fetish} in sameSexualQuirk(). Please report this.`;
			}
		}
	}

	function newChildFetish(fetish = "none") {
		switch (fetish) {
			case "submissive":
				return `is now sexually submissive`;
			case "cumslut":
				return `is now a cumslut`;
			case "humiliation":
				return `now has a fetish for humiliation`;
			case "buttslut":
				return `is now a buttslut`;
			case "boobs":
				return `now has a fetish for all things breast-related`;
			case "sadist":
				return `now gets off on causing pain`;
			case "masochist":
				return `now gets off on pain`;
			case "dom":
				return `is now very sexually dominant`;
			case "pregnancy":
				return `has developed a fascination for all things pregnancy-related`;
			default:
				throw `<span class="error">Unexpected fetish value of "${fetish}" in newChildFetish(). Please report this.</span>`;
		}
	}

	function childGrowTime(child) {
		const
			nameSpan = App.UI.DOM.makeElement("span", child.slaveName, "pink"),
			limeSpan = App.UI.DOM.makeElement("span", 'ready for release.', "lime"),
			mainSpan = document.createElement("span"),

			{ He } = getPronouns(child);

		if (child.growTime > 0) {
			mainSpan.append(nameSpan, ` is growing steadily. ${He} will be ready for release in about ${years(child.growTime)}. `);
		} else {
			mainSpan.append(nameSpan, ' is ', limeSpan, ` ${He} will be removed from ${V.nurseryName} upon your approach. `);
		}

		return mainSpan;
	}

	function findMedianNannyIntelligence() {
		return median(nannies.map(n => n.intelligence));
	}

	function findMedianNannyIntelligenceImplant() {
		return median(nannies.map(n => n.intelligenceImplant));
	}

	/**
	 * Checks if there are more than one children over the target age in the Nursery
	 * @param {number} index
	 * @param {number} age=3
	 */
	function multipleChildrenOverTargetAge(index, age = 3) {
		const cribsCopy = Array.from(V.cribs);

		cribsCopy.splice(index, 1);

		return V.cribs.length > 1 && cribsCopy.some(c => c.actualAge >= age);
	}

	return frag;
};
