/* Note that we use a much more strict delineation between individual and nonindividual events here than in the old event system.
 * Individual events always trigger for the chosen event slave, and the first actor is always the event slave.
 * Nonindividual events are not provided any event slave and should cast one themselves.
 */

/** get a list of possible individual events
 * @returns {Array<App.Events.BaseEvent>}
 */
App.Events.getIndividualEvents = function() {
	return [
		// instantiate all possible random individual events here
		// example: new App.Events.TestEvent(),
		new App.Events.RESSAssFitting(),
		new App.Events.RESSCockFeederResistance(),
		new App.Events.RESSComfortableSeat(),
		new App.Events.RESSDevotedAnalVirgin(),
		new App.Events.RESSDevotedEducated(),
		new App.Events.RESSDevotedShortstack(),
		new App.Events.RESSDevotedVirgin(),
		new App.Events.RESSDevotedWaist(),
		new App.Events.RESSEscapee(),
		new App.Events.RESSFrighteningDick(),
		new App.Events.RESSHotPC(),
		new App.Events.RESSImScared(),
		new App.Events.RESSKitchenMolestation(),
		new App.Events.RESSLazyEvening(),
		new App.Events.RESSMoistPussy(),
		new App.Events.RESSMuscles(),
		new App.Events.RESSObedientAddict(),
		new App.Events.RESSObedientBitchy(),
		new App.Events.RESSObedientGirlish(),
		new App.Events.RESSObedientIdiot(),
		new App.Events.RESSObedientShemale(),
		new App.Events.RESSPassingDeclaration(),
		new App.Events.RESSRetchingCum(),
		new App.Events.RESSServeThePublicDevoted(),
		new App.Events.RESSSlaveOnSlaveClit(),
		new App.Events.RESSSlaveOnSlaveDick(),
		new App.Events.RESSSuppositoryResistance(),
		new App.Events.RESSTooThinForCumDiet(),
		new App.Events.RESSWaistlineWoes(),

		new App.Events.RECIButthole(),
		new App.Events.RECIFeminization(),
		new App.Events.RECIFuta(),
		new App.Events.RECIMilf(),
		new App.Events.RECIOrientation(),
		new App.Events.RECIUgly(),

		new App.Events.RETSSiblingTussle(),
		new App.Events.RETSSimpleAssault(),
	];
};

/** get a list of possible nonindividual events
 * @returns {Array<App.Events.BaseEvent>}
 */
App.Events.getNonindividualEvents = function() {
	return [
		// instantiate all possible random nonindividual events here
		// example: new App.Events.TestEvent(),
		new App.Events.REDevotees(),
		new App.Events.RERelativeRecruiter(),
		new App.Events.REStaffedMorning(),
	];
};

/** choose a valid, castable event from the given event list
 * @param {Array<App.Events.BaseEvent>} eventList - list of events to filter
 * @param {App.Entity.SlaveState} [slave] - event slave (mandatory to cast in first actor slot).  omit for nonindividual events.
 * @returns {Array<App.Events.BaseEvent>}
 */
App.Events.getValidEvents = function(eventList, slave) {
	return eventList
		.filter(e => (e.eventPrerequisites().every(p => p()) && e.castActors(slave)))
		.reduce((res, cur) => res.concat(Array(cur.weight).fill(cur)), []);
};


/* --- below here is a bunch of workaround crap because we have to somehow persist event selection through multiple twine passages. ---
 * eventually all this should go away, and we should use just one simple passage for both selection and execution, so everything can be kept in object form instead of being continually serialized and deserialized.
 * we need to be able to serialize/deserialize the active event anyway so that saves work right, so this mechanism just piggybacks on that capability so the event passages don't need to be reworked all at once
 */

/** get a stringified list of possible individual events as fake passage names - TODO: kill me */
App.Events.getIndividualEventsPassageList = function(slave) {
	const events = App.Events.getValidEvents(App.Events.getIndividualEvents(), slave);
	return events.map(e => `JSRE ${e.eventName}:${JSON.stringify(e.toJSON())}`);
};

/** get a stringified list of possible individual events as fake passage names - TODO: kill me */
App.Events.getNonindividualEventsPassageList = function() {
	const events = App.Events.getValidEvents(App.Events.getNonindividualEvents());
	return events.map(e => `JSRE ${e.eventName}:${JSON.stringify(e.toJSON())}`);
};

/** execute a fake event passage from the embedded JSON - TODO: kill me */
App.Events.setGlobalEventForPassageTransition = function(psg) {
	V.event = JSON.parse(psg.slice(psg.indexOf(":") + 1));
};

/** strip the embedded JSON from the fake event passage so it can be read by a human being - TODO: kill me */
App.Events.printEventPassage = function(psg) {
	return psg.slice(0, psg.indexOf(":"));
};
