/**
 * Displays a value that you can click on to get some details in an overlay
 */
App.UI.DOM.InteractiveDetails = class {
	/**
	 * @param {string} linkText text to use for the show/hide link
	 * @param {function(): HTMLElement|DocumentFragment} [detailsGenerator] function which generates the contents of the details overlay (omit to disable details)
	 * @param {string[]} [linkClasses=[]] list of extra CSS classes to apply to the link
	 */
	constructor(linkText, detailsGenerator, linkClasses=[]) {
		this.span = App.UI.DOM.makeElement("span", "", "details-overlay");
		this.span.style.visibility = "hidden";
		this.link = detailsGenerator ? App.UI.DOM.link(linkText, () => this.toggle()) : App.UI.DOM.makeElement("span", linkText);
		this.link.classList.add(...linkClasses);
		this.func = detailsGenerator;
		this.shown = false;
	}

	/**
	 * Toggle the visibility of the overlay (changing the render state)
	 */
	toggle() {
		this.shown = !this.shown;
		if (this.shown) {
			$(this.span).empty().append(this.func());
			// by default, the CSS will try to show it as inline-block on the right
			this.span.style.removeProperty("right"); // reset style in case we'd moved it last time it was shown
			const overlayRect = this.span.getBoundingClientRect();
			const linkRect = this.link.getBoundingClientRect();
			if (overlayRect.right > window.innerWidth) {
				if (overlayRect.width < linkRect.left) {
					// if it won't fit in the default position, but it *will* fit on the left, force it to move over to the left side of the link
					this.span.style.right = "100%";
				}
				// if it won't fit on either side, give up and leave it where it is.  TODO: maybe do something smarter?
			}
			this.span.style.visibility = "visible";
		} else {
			$(this.span).empty();
			this.span.style.visibility = "hidden";
		}
	}

	/**
	 * Render the object to the page
	 * @returns {HTMLSpanElement}
	 */
	render() {
		const containingSpan = document.createElement("span");
		containingSpan.style.position = "relative"; // required for absolute positioning to work correctly later
		containingSpan.append(this.link, this.span);
		return containingSpan;
	}
};
