App.UI.MultipleInspect = (function() {
	/**
	 * Provide a mechanism to inspect multiple slaves at once (for example, for Household Liquidators and recETS).
	 * Intended for use from DOM passages.
	 * @param {Array<App.Entity.SlaveState>} slaves
	 * @param {boolean} showFamilyTree
	 * @param {string} [market]
	 * @returns {DocumentFragment}
	 */
	function MultipleInspectDOM(slaves, showFamilyTree, market) {
		const frag = document.createDocumentFragment();
		const tabBar = App.UI.DOM.appendNewElement("div", frag, "", "tab-bar");

		for (const slave of slaves) {
			tabBar.append(App.UI.tabBar.tabButtonDOM(`slave${slave.ID}`, slave.slaveName));
			frag.append(App.UI.tabBar.makeTabDOM(`slave${slave.ID}`, App.Desc.longSlave(slave, {market: market})));
		}

		if (slaves.length > 1 && showFamilyTree) {
			const button = App.UI.tabBar.tabButtonDOM(`familyTreeTab`, "Family Tree");
			button.addEventListener('click', event => {
				renderFamilyTree(slaves, slaves[0].ID);
			});
			tabBar.append(button);
			const ftTarget = document.createElement("div");
			ftTarget.setAttribute("id", "family-tree");
			frag.append(App.UI.tabBar.makeTabDOM(`familyTreeTab`, ftTarget));
		}

		App.UI.tabBar.handlePreSelectedTab(`slave${slaves[0].ID}`);

		return frag;
	}

	return MultipleInspectDOM;
})();
