/**
 * @param {App.Entity.PlayerState|App.Entity.SlaveState} entity
 * @param {boolean} player
 * @returns {HTMLDivElement}
 */
App.Medicine.Modification.eyeSelector = function(entity, player = false) {
	const {He, him, his} = getPronouns(entity);

	let selectedSide = "none";
	let selectedIris = "none";
	let selectedPupil = "none";
	let selectedSclera = "none";

	let removeDiv = document.createElement("div");
	removeDiv.classList.add("choices");
	let applyDiv = document.createElement("div");

	const container = document.createElement("div");
	container.append(
		`${player ? "You have" : `${He} has`} ${App.Desc.eyesColorLong(entity)}, ${hasBothEyes(
			entity) ? "they are" : "it is"} ${App.Desc.eyesType(entity)}.`,
		removeDiv, "You have a number of contact lenses in various colors available. ",
		App.UI.DOM.makeElement("span", `You can change what ${player ? "your" : his} eyes look like.`, "note"),
		assembleLinks(), applyDiv
	);
	updateRemoveDiv();
	updateApplyDiv();
	return container;

	function assembleLinks() {
		const sides = ["left", "right", "both"];
		const irisColors = App.Medicine.Modification.eyeColor.map(color => color.value);
		const pupilShapes = ["none"].concat(App.Medicine.Modification.eyeShape.map(shape => shape.value));
		const scleraColors = ["none"].concat(App.Medicine.Modification.eyeColor.map(color => color.value));
		const div = document.createDocumentFragment();
		div.append(
			assembleList("Side: ", sides, value => selectedSide = value, selectedIris),
			assembleList("Iris: ", irisColors, value => selectedIris = value, selectedSide),
			assembleList("Pupil: ", pupilShapes, value => selectedPupil = value, selectedPupil),
			assembleList("Sclera: ", scleraColors, value => selectedSclera = value, selectedSclera)
		);
		return div;
	}

	/**
	 * @param {string} name
	 * @param {Array<string>} list
	 * @param {Function}callback
	 * @param {string} selected
	 * @returns {HTMLDivElement}
	 */
	function assembleList(name, list, callback, selected) {
		const links = [];

		for (let i = 0; i < list.length; i++) {
			addToggle(list[i], callback, links, list[i] === selected);
		}

		const div = document.createElement("div");
		div.classList.add("choices");
		div.append(name, App.UI.DOM.arrayToList(links, " | ", " | "));
		return div;
	}

	/**
	 * @param {string} value
	 * @param {Function} callback
	 * @param {Array<HTMLAnchorElement>} links
	 * @param {boolean} [disabled]
	 */
	function addToggle(value, callback, links, disabled = false) {
		const a = document.createElement("a");
		a.append(capFirstChar(value));
		if (disabled) {
			a.classList.add("disabled");
		}
		a.onclick = () => {
			for (let link of links) {
				link.classList.remove("disabled");
			}
			a.classList.add("disabled");
			callback(value);
			updateRemoveDiv();
			updateApplyDiv();
		};
		links.push(a);
	}

	function updateApplyDiv() {
		$(applyDiv).empty();
		if (selectedSide !== "none" && selectedIris !== "none") {
			// make the following easier to read
			let both = selectedSide === "both";
			let leftGlass = !hasLeftEye(entity) || getLeftEyeType(entity) === 2;
			let rightGlass = !hasRightEye(entity) || getRightEyeType(entity) === 2;

			// base eye
			let r = player ? "" : ` ${him}`;
			if (both) {
				if (leftGlass && rightGlass) {
					r += ` ${selectedIris} glass eyes`;
				} else if (leftGlass || rightGlass) {
					r += ` a glass eye and a ${selectedIris} lens`;
				} else {
					r += ` ${selectedIris} lenses`;
				}
			} else {
				r += " a";
				if ((selectedSide === "left" && leftGlass) || (selectedSide === "right" && rightGlass)) {
					r += ` ${selectedIris} glass eye`;
				} else {
					r += ` ${selectedIris} lens`;
				}
			}
			// pupil & sclera
			if (selectedPupil !== "none" || selectedSclera !== "none") {
				r += " with";
				if (selectedPupil !== "none") {
					r += ` ${both ? selectedPupil : addA(selectedPupil)}`;
					if (both) {
						r += " pupils";
					} else {
						r += " pupil";
					}
					if (selectedSclera !== "none") {
						r += " and";
					}
				}
				if (selectedSclera !== "none") {
					r += ` ${selectedSclera}`;
					if (both) {
						r += " sclerae";
					} else {
						r += " sclera";
					}
				}
			}
			if (!both) {
				r += ` for ${player ? "your" : his} ${selectedSide} eye`;
			}
			r += "?";

			const a = document.createElement("a");
			a.append(player ? "Take" : "Give");
			a.onclick = applyLink;
			applyDiv.append(a, r);
			if (!player) {
				applyDiv.append(" ",
					App.UI.DOM.makeElement("span", "This is independent from eyewear choices.", "note"));
			}
		}
	}

	function applyLink() {
		// make sure the eye exists; give glass eye if there is none
		if ((selectedSide === "left" || selectedSide === "both") && getLeftEyeType(entity) === 0) {
			eyeSurgery(entity, "left", "glass");
		}
		if ((selectedSide === "right" || selectedSide === "both") && getRightEyeType(entity) === 0) {
			eyeSurgery(entity, "right", "glass");
		}

		// apply modifications
		setEyeColorFull(entity, selectedIris,
			selectedPupil === "none" ? "" : selectedPupil,
			selectedSclera === "none" ? "" : selectedSclera,
			selectedSide);
		cashX(forceNeg(V.modCost), "slaveMod", entity);

		App.UI.reload();
	}

	function updateRemoveDiv() {
		$(removeDiv).empty();
		const links = [];
		let _n = 0;
		// remove lenses
		if (hasLeftEye(entity) && getLeftEyeColor(entity) !== getGeneticEyeColor(entity, "left")) {
			_n++;
			links.push(removeLink("Remove left lens", () => resetEyeColor(entity, "left")));
		}
		if (hasRightEye(entity) && getRightEyeColor(entity) !== getGeneticEyeColor(entity, "right")) {
			_n++;
			links.push(removeLink("Remove right lens", () => resetEyeColor(entity, "right")));
		}
		if (_n === 2) {
			links.push(removeLink("Remove both lenses", () => resetEyeColor(entity, "both")));
		}
		// remove glass eyes
		_n = 0;
		if (getLeftEyeType(entity) === 2) {
			_n++;
			links.push(removeLink("Remove left glass eye", () => eyeSurgery(entity, "left", "remove")));
		}
		if (getRightEyeType(entity) === 2) {
			_n++;
			links.push(removeLink("Remove right glass eye", () => eyeSurgery(entity, "right", "remove")));
		}
		if (_n === 2) {
			links.push(removeLink("Remove both glass eyes", () => eyeSurgery(entity, "both", "remove")));
		}
		if (links.length > 0) {
			removeDiv.append(App.UI.DOM.arrayToList(links, " | ", " | "));
		}
	}

	function removeLink(text, callback) {
		const a = document.createElement("a");
		a.append(text);
		a.onclick = () => {
			callback();
			App.UI.reload();
		};
		return a;
	}
};

/**
 * Update ears in salon
 * @param {App.Entity.SlaveState} slave
 * @param {object} params
 * @param {number|string} [params.primaryEarColor]
 * @param {string} [params.secondaryEarColor]
 * @returns {node}
 */
App.Medicine.Salon.ears = function(slave, {primaryEarColor = 0, secondaryEarColor = ""} = {}) {
	const frag = new DocumentFragment();
	let updatePrimary = (newVal) => { primaryEarColor = newVal; apply(); };
	let updateSecondary = (newVal) => { secondaryEarColor = newVal; apply(); };

	if (slave.earT !== "none" && slave.earTColor !== "hairless") {
		const {His, his} = getPronouns(slave);
		let div;
		let p;
		frag.append(`${His} fluffy ears are ${slave.earTColor}.`);

		div = document.createElement("div");
		div.classList.add("choices");
		if (slave.earTColor !== slave.hColor) {
			div.append(
				App.UI.DOM.link(
					"Match current hair",
					() => {
						slave.earTColor = slave.hColor;
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						apply();
					}
				)
			);
			div.append(" or ");
			App.UI.DOM.appendNewElement("span", div, "choose a new one: ", "note");
		} else {
			App.UI.DOM.appendNewElement("span", div, `Choose a dye color before dyeing ${his} ears:`, "note");
		}
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Colors:`);
		div.append(createList(App.Medicine.Modification.Color.Primary, updatePrimary));
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Highlights:`);
		div.append(createList(App.Medicine.Modification.Color.Secondary, updateSecondary));
		frag.append(div);

		if (primaryEarColor !== 0) {
			p = document.createElement("p");
			p.classList.add("choices");
			p.append(
				App.UI.DOM.link(
					`Color ${his} ears`,
					() => {
						slave.earTColor = (primaryEarColor + secondaryEarColor);
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						cashX(forceNeg(V.modCost), "slaveMod", slave);
						App.Medicine.Salon.ears(slave); // discard selections after locking them in.
					}
				)
			);
			p.append(` ${primaryEarColor}${secondaryEarColor} now?`);
			frag.append(p);
		}
	}
	return jQuery("#salon-ears").empty().append(frag);

	function createList(array, method) {
		const links = [];
		for (const item of array) {
			const title = item.title || capFirstChar(item.value);
			links.push(
				App.UI.DOM.link(
					title,
					() => method(item.value)
				)
			);
		}
		return App.UI.DOM.generateLinksStrip(links);
	}
	function apply() {
		App.Art.refreshSlaveArt(slave, 3, "art-frame");
		App.Medicine.Salon.ears(
			slave,
			{
				primaryEarColor: primaryEarColor,
				secondaryEarColor: secondaryEarColor,
			}
		);
	}
};

/**
 * Update hair in salon
 * @param {App.Entity.SlaveState} slave
 * @param {object} params
 * @param {number|string} [params.primaryHairColor]
 * @param {string} [params.secondaryHairColor]
 * @returns {node}
 */
App.Medicine.Salon.hair = function(slave, {primaryHairColor = 0, secondaryHairColor = ""} = {}) {
	const frag = new DocumentFragment();
	let updatePrimary = (newVal) => { primaryHairColor = newVal.value; apply(); };
	let updateSecondary = (newVal) => { secondaryHairColor = newVal.value; apply(); };
	const {His, his, He, him} = getPronouns(slave);

	if (slave.bald !== 1) {
		frag.append(hairDye());
		frag.append(hairStyle());
		frag.append(hairLength());
		frag.append(hairMaint());
	} else {
		// Bald
		if (slave.hStyle === "bald") {
			frag.append(`${He} is completely bald. `);
		} else {
			frag.append(wigDye());
		}
		frag.append(wigStyle());
		frag.append(wigLength());
	}
	return jQuery("#salon-hair").empty().append(frag);

	function hairDye() {
		const frag = new DocumentFragment();
		let div;
		let p;
		frag.append(`${His} hair is ${slave.hColor}.`);

		div = document.createElement("div");
		div.classList.add("choices");
		if (slave.origHColor !== slave.hColor) {
			div.append(
				App.UI.DOM.link(
					"Restore natural color",
					() => {
						slave.hColor = slave.origHColor;
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						cashX(forceNeg(V.modCost), "slaveMod", slave);
						apply();
					}
				)
			);
			div.append(" or ");
			App.UI.DOM.appendNewElement("span", div, "choose a new one: ", "note");
		} else {
			App.UI.DOM.appendNewElement("span", div, `Choose a dye color before dyeing ${his} hair:`, "note");
		}
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Colors:`);
		div.append(createList(App.Medicine.Modification.Color.Primary, updatePrimary));
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Highlights:`);
		div.append(createList(App.Medicine.Modification.Color.Secondary, updateSecondary));
		frag.append(div);

		if (primaryHairColor !== 0) {
			p = document.createElement("p");
			p.classList.add("choices");
			p.append(
				App.UI.DOM.link(
					`Color ${his} hair`,
					() => {
						slave.hColor = (primaryHairColor + secondaryHairColor);
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						cashX(forceNeg(V.modCost), "slaveMod", slave);
						App.Medicine.Salon.hair(slave); // discard selections after locking them in.
					}
				)
			);
			p.append(` ${primaryHairColor}${secondaryHairColor} now?`);
			frag.append(p);
		}
		return frag;
	}

	function hairStyle() {
		const frag = new DocumentFragment();
		let div;
		let method;
		div = document.createElement("div");
		if (slave.hStyle !== "shaved") {
			div.append(`${His} ${slave.hStyle} hair is ${lengthToEitherUnit(slave.hLength)} long. `);
		} else {
			div.append(`${His} hair is shaved smooth. `);
		}
		App.UI.DOM.appendNewElement("span", div, `General hairstyles will conform to hair length and clothing choices.`, "note");
		frag.append(div);

		// Normal styles
		div = document.createElement("div");
		div.classList.add("choices");
		method = (newVal) => {
			slave.hStyle = newVal.value;
			cashX(forceNeg(V.modCost), "slaveMod", slave);
			apply();
		};
		if (slave.hLength > 1) {
			div.append(`Style ${his} hair:`);
			div.append(createList(App.Medicine.Modification.hairStyles.Normal, method));
		} else {
			App.UI.DOM.appendNewElement("span", div, `${His} hair is too short to style meaningfully`, "note");
		}
		frag.append(div);

		// Short styles, includes cutting
		div = document.createElement("div");
		div.classList.add("choices");
		method = (newVal) => {
			slave.hStyle = newVal.value;
			slave.hLength = newVal.hLength;
			cashX(forceNeg(V.modCost), "slaveMod", slave);
			apply();
		};
		if (slave.hLength > 1) {
			div.append(`Cut and style ${his} hair:`);
			div.append(createList(App.Medicine.Modification.hairStyles.Cut, method));
		}
		frag.append(div);

		return frag;
	}

	function hairLength() {
		const frag = new DocumentFragment();
		let div = document.createElement("div");
		div.classList.add("choices");
		let method = (newVal) => {
			if (newVal.hasOwnProperty("onApplication")) {
				newVal.onApplication(slave);
			}
			if (newVal.hasOwnProperty("hLength")) {
				slave.hLength = newVal.hLength;
			}
			apply();
		};
		const oldHLength = (V.showInches === 2) ? Math.round(slave.hLength / 2.54) : slave.hLength;

		App.UI.DOM.appendNewElement("span", div, `Cut or lengthen ${his} hair:`);
		div.append(createList(App.Medicine.Modification.hairStyles.Length, method));
		div.append(" | Custom length: ");
		div.append(
			App.UI.DOM.makeTextBox(
				oldHLength,
				v => {
					v = Math.max(v, 0); // Positive hair length only
					// If they entered "inches," convert
					if (V.showInches === 2) {
						v = Math.round(v * 2.54);
					}
					slave.hLength = v;
					cashX(forceNeg(V.modCost), "slaveMod", slave);
					apply();
				},
				true
			)
		);
		if (V.showInches === 1) {
			div.append(`cm (${cmToInchString(slave.hLength)})`);
		} else if (V.showInches === 2) {
			div.append(`inches`);
		}

		frag.append(div);

		return frag;
	}

	function hairMaint() {
		let div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Have ${his} hair carefully maintained at its current length: `);
		let haircuts;
		let text;
		if (slave.haircuts === 1) {
			text = "Cease maintenance";
			haircuts = 0;
		} else {
			text = "Begin maintenance";
			haircuts = 1;
		}
		div.append(
			App.UI.DOM.link(
				text,
				() => {
					slave.haircuts = haircuts;
					apply();
				}
			)
		);
		return div;
	}

	function wigDye() {
		const frag = new DocumentFragment();
		let div;
		let p;
		frag.append(`${His} current wig is ${slave.hColor}. `);

		if (slave.hStyle !== "bald") {
			frag.append(
				App.UI.DOM.link(
					"Remove wig",
					() => {
						slave.hStyle = "bald";
						slave.hLength = 0;
						// I'm not going to charge you for taking off a fucking wig.
						apply();
					}
				)
			);
			frag.append(" or ");
			App.UI.DOM.appendNewElement("span", frag, "choose a new one: ", "note");
		} else {
			App.UI.DOM.appendNewElement("span", frag, `Choose a wig color:`, "note");
		}

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Colors:`);
		div.append(createList(App.Medicine.Modification.Color.Primary, updatePrimary));
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Highlights:`);
		div.append(createList(App.Medicine.Modification.Color.Secondary, updateSecondary));
		frag.append(div);

		if (primaryHairColor !== 0) {
			p = document.createElement("p");
			p.classList.add("choices");
			p.append(
				App.UI.DOM.link(
					`Change`,
					() => {
						slave.earTColor = (primaryHairColor + secondaryHairColor);
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						cashX(forceNeg(V.modCost), "slaveMod", slave);
						App.Medicine.Salon.hair(slave); // discard selections after locking them in.
					}
				)
			);
			p.append(` ${his} wig color to ${primaryHairColor}${secondaryHairColor} now?`);
			frag.append(p);
		}
		return frag;
	}

	function wigLength() {
		const frag = new DocumentFragment();
		if (slave.hStyle === "bald") {
			return frag;
		}
		let div = document.createElement("div");
		div.classList.add("choices");
		const array = [];
		for (const number of [10, 30, 60, 90, 120, 150]) {
			const obj = {};
			obj.title = lengthToEitherUnit(number);
			obj.hLength = number;
			array.push(obj);
		}
		let method = (newVal) => {
			slave.hLength = newVal.hLength;
			apply();
		};
		const oldHLength = (V.showInches === 2) ? Math.round(slave.hLength / 2.54) : slave.hLength;
		App.UI.DOM.appendNewElement("span", div, `Set wig length to:`, "choices");
		div.append(createList(array, method));
		div.append(" | Custom length: ");
		div.append(
			App.UI.DOM.makeTextBox(
				oldHLength,
				v => {
					v = Math.max(v, 10); // Wigs must be at least 10 cm
					// If they entered "inches," convert
					if (V.showInches === 2) {
						v = Math.round(v * 2.54);
					}
					slave.hLength = v;
					cashX(forceNeg(V.modCost), "slaveMod", slave);
					apply();
				},
				true
			)
		);
		if (V.showInches === 1) {
			div.append(`cm (${cmToInchString(slave.hLength)})`);
		} else if (V.showInches === 2) {
			div.append(`inches`);
		}

		frag.append(div);

		return frag;
	}

	function wigStyle() {
		const frag = new DocumentFragment();
		let div = document.createElement("div");
		div.classList.add("choices");
		const method = (newVal) => {
			slave.hStyle = newVal.value;
			cashX(forceNeg(V.modCost), "slaveMod", slave);
			apply();
		};

		if (slave.hStyle !== "bald") {
			frag.append(`${His} ${slave.hStyle} wig is ${lengthToEitherUnit(slave.hLength)} long. `);
		} else {
			frag.append(`${He} is not wearing a wig. `);
		}
		App.UI.DOM.appendNewElement("span", frag, `General hairstyles will conform to hair length and clothing choices.`, "note");

		div = document.createElement("div");
		div.classList.add("choices");
		if (slave.hStyle === "bald") {
			div.append(`Give ${him} a wig:`);
		} else {
			div.append(`Set wig style:`);
		}
		div.append(createList(App.Medicine.Modification.hairStyles.Normal, method));
		frag.append(div);
		return frag;
	}

	function createList(array, method) {
		const links = [];
		for (const item of array) {
			if (item.hasOwnProperty("requirements")) {
				if (item.requirements(slave) === false) {
					continue;
				}
			}
			const title = item.title || capFirstChar(item.value);
			links.push(
				App.UI.DOM.link(
					title,
					() => method(item)
				)
			);
		}
		return App.UI.DOM.generateLinksStrip(links);
	}

	function apply() {
		App.Art.refreshSlaveArt(slave, 3, "art-frame");
		App.Medicine.Salon.hair(
			slave,
			{
				primaryHairColor: primaryHairColor,
				secondaryHairColor: secondaryHairColor,
			}
		);
	}
};

/**
 * Update hair in salon
 * @param {App.Entity.SlaveState} slave
 * @param {object} params
 * @param {number|string} [params.primaryTailColor]
 * @param {string} [params.secondaryTailColor]
 * @returns {node}
 */
App.Medicine.Salon.tail = function(slave, {primaryTailColor = 0, secondaryTailColor = ""} = {}) {
	const frag = new DocumentFragment();
	let updatePrimary = (newVal) => { primaryTailColor = newVal.value; apply(); };
	let updateSecondary = (newVal) => { secondaryTailColor = newVal.value; apply(); };
	const {His, his} = getPronouns(slave);

	if (slave.tail !== "none") {
		frag.append(tailDye());
	}

	return jQuery("#salon-tail").empty().append(frag);

	function tailDye() {
		const frag = new DocumentFragment();
		let div;
		let p;
		frag.append(`${His} tail is ${slave.tailColor}.`);

		div = document.createElement("div");
		div.classList.add("choices");
		if (slave.origHColor !== slave.hColor) {
			div.append(
				App.UI.DOM.link(
					"Match current hair",
					() => {
						slave.tailColor = slave.hColor;
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						cashX(forceNeg(V.modCost), "slaveMod", slave);
						apply();
					}
				)
			);
			div.append(" or ");
			App.UI.DOM.appendNewElement("span", div, "choose a new one: ", "note");
		} else {
			App.UI.DOM.appendNewElement("span", div, `Choose a dye color before dyeing ${his} tail:`, "note");
		}
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Colors:`);
		div.append(createList(App.Medicine.Modification.Color.Primary, updatePrimary));
		frag.append(div);

		div = document.createElement("div");
		div.classList.add("choices");
		div.append(`Highlights:`);
		div.append(createList(App.Medicine.Modification.Color.Secondary, updateSecondary));
		frag.append(div);

		if (primaryTailColor !== 0) {
			p = document.createElement("p");
			p.classList.add("choices");
			p.append(
				App.UI.DOM.link(
					`Color ${his} tail`,
					() => {
						slave.tailColor = (primaryTailColor + secondaryTailColor);
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
						cashX(forceNeg(V.modCost), "slaveMod", slave);
						App.Medicine.Salon.tail(slave); // discard selections after locking them in.
					}
				)
			);
			p.append(` ${primaryTailColor}${secondaryTailColor} now?`);
			frag.append(p);
		}
		return frag;
	}

	function createList(array, method) {
		const links = [];
		for (const item of array) {
			if (item.hasOwnProperty("requirements")) {
				if (item.requirements(slave) === false) {
					continue;
				}
			}
			const title = item.title || capFirstChar(item.value);
			links.push(
				App.UI.DOM.link(
					title,
					() => method(item)
				)
			);
		}
		return App.UI.DOM.generateLinksStrip(links);
	}

	function apply() {
		App.Art.refreshSlaveArt(slave, 3, "art-frame");
		App.Medicine.Salon.tail(
			slave,
			{
				primaryTailColor: primaryTailColor,
				secondaryTailColor: secondaryTailColor,
			}
		);
	}
};
