/**
 * @param {App.Entity.SlaveState} slave
 * @param {object} params
 * @param {string|number} [params.market] 0 if the slave is not for sale.  Otherwise a string with the name of the market,
 * partially to determine if laws apply to the market or not.
 * @param {number} [params.eventDescription]
 * @param {string} [params.prisonCrime]
 * @returns {DocumentFragment}
 */
App.Desc.longSlave = function(slave = V.activeSlave, {market = 0, eventDescription = 0, prisonCrime} = {}) {
	const {
		He, His, him, he, his
	} = getPronouns(slave);
	let el = new DocumentFragment();
	let span;
	let frag;
	let p;
	let r;
	const applyLaw = applyLawCheck(market);
	SlaveStatClamp(slave);

	if (V.seeImages === 1 && !eventDescription && passage() !== "Slave Interact") {
		// Art
		span = document.createElement('span');
		span.id = "art-frame";
		App.Events.drawEventArt(span, slave);
		el.appendChild(span);
	}

	p = document.createElement("p");

	// Name
	p.appendChild(
		App.UI.DOM.makeElement(
			"span",
			`${SlaveFullName(slave)} `,
			["slave", "name", "simple"]
		)
	);

	// Label
	if (slave.custom.label) {
		frag = new DocumentFragment();
		frag.append("(");
		span = document.createElement('span');
		span.classList.add("custom-label");
		span.textContent = slave.custom.label;
		frag.append(span);
		frag.append(") ");
		p.appendChild(frag);
	}

	if (market && V.ui !== "start") {
		if (applyLaw === 1) {
			p.append(`has passed inspection to be sold in your arcology. `);
			$(p).append(App.Desc.lawCompliance(slave, market)); // includes CheckForGingering
			p.append(` `);
		} else {
			p.append(`is for sale and is available to inspect. `);
			if (prisonCrime) {
				// reports a slave's crime in the criminal market
				p.append(`${He} ${prisonCrime} `);
			}
		}
		el.appendChild(p);

		p = document.createElement("p");
		p.className = "indent";
		p.appendChild(
			App.UI.DOM.makeElement(
				"span",
				`${slave.slaveName} `,
				"name"
			)
		);
	}

	p.append(`is `);
	// Devotion
	frag = new DocumentFragment();
	span = document.createElement('span');

	if (slave.devotion < -95) {
		frag.append("a ");
		span.className = "devotion hateful";
		span.textContent = "hate-filled, ";
	} else if (slave.devotion < -50) {
		frag.append("a ");
		span.className = "devotion hateful";
		span.textContent = "hateful, ";
	} else if (slave.devotion < -20) {
		frag.append("a ");
		span.className = "devotion resistant";
		span.textContent = "reluctant, ";
	} else if (slave.devotion <= 20) {
		frag.append("a ");
		span.className = "devotion ambivalent";
		span.textContent = "hesitant, ";
	} else if (slave.devotion <= 50) {
		frag.append("an ");
		span.className = "devotion accept";
		span.textContent = "accepting, ";
	} else if (slave.devotion <= 95) {
		frag.append("a ");
		span.className = "devotion devoted";
		span.textContent = "devoted, ";
	} else {
		frag.append("a ");
		span.className = "devotion worship";
		span.textContent = "worshipful, ";
	}

	frag.appendChild(span);
	p.appendChild(frag);

	// Trust
	span = document.createElement('span');
	if (slave.trust < -95) {
		span.className = "trust terrified";
		span.textContent = "abjectly terrified ";
	} else if (slave.trust < -50) {
		span.className = "trust terrified";
		span.textContent = "terrified ";
	} else if (slave.trust < -20) {
		span.className = "trust frightened";
		span.textContent = "frightened ";
	} else if (slave.trust < 20) {
		span.className = "trust fearful";
		span.textContent = "fearful ";
	} else if (slave.trust <= 50) {
		if (slave.devotion < -20) {
			span.className = "defiant careful";
			span.textContent = "careful ";
		} else {
			span.className = "trust careful";
			span.textContent = "careful ";
		}
	} else if (slave.trust < 95) {
		if (slave.devotion < -20) {
			span.className = "defiant bold";
			span.textContent = "bold ";
		} else {
			span.className = "trust trusting";
			span.textContent = "trusting ";
		}
	} else {
		if (slave.devotion < -20) {
			span.className = "defiant full";
			span.textContent = "defiant ";
		} else {
			span.className = "trust prof-trusting";
			span.textContent = "profoundly trusting ";
		}
	}
	p.appendChild(span);

	// Slave's Title, ex: "pregnant big bottomed busty milky hourglass broodmother"
	span = document.createElement('span');
	span.style.fontWeight = "bold";
	span.className = "coral";
	jQuery(span).append(`${SlaveTitle(slave)}. `);
	p.appendChild(span);

	r = [];
	// Indenture
	if (slave.indenture > -1) {
		r.push(`${His}`);
		if (slave.indentureRestrictions > 1) {
			r.push(`restrictive`);
		} else if (slave.indentureRestrictions > 0) {
			r.push(`protective`);
		} else {
			r.push(`unrestricted`);
		}
		r.push(`indenture`);
		if (slave.indenture > 0) {
			if (slave.indenture > 1) {
				r.push(`has ${slave.indenture} weeks left to run.`);
			} else {
				r.push(`expires next week.`);
			}
		} else {
			r.push(`expires this week.`);
		}
	}

	r.push(
		App.Desc.sceneIntro(slave, {
			market: market, eventDescription: eventDescription
		})
	);
	r.push(App.Desc.name(slave));
	r.push(App.Desc.ageAndHealth(slave));
	$(p).append(r.join(" "));

	r = []; const clinicNameCaps = capFirstChar(V.clinicName);
	if (!market) {
		if (V.clinic !== 0 && V.clinicUpgradeScanner === 1) {
			if (slave.chem > 15) {
				$(p).append(`${clinicNameCaps}'s scanners score long term carcinogenic buildup in ${his} body at `);
				span = document.createElement('span');
				span.className = "cyan";
				span.textContent = `${Math.ceil(slave.chem / 10)}.`;
				p.append(span);
			} else {
				p.append(`${clinicNameCaps}'s scanners confirm that ${he} has good prospects for long term health. `);
			}
		}

		r.push(App.Desc.geneticQuirkAssessment(slave));

		if (V.showSexualHistory === 1 && V.ui !== "start") {
			r.push(App.Desc.sexualHistory(slave));
		}
	}

	r.push(App.Desc.mind(slave, {market: market}));

	if (!market) {
		if (eventDescription === 0) {
			if (canSee(slave)) {
				if (slave.attrKnown === 1) {
					if ((slave.assignment !== "work in the dairy") || (V.dairyRestraintsSetting > 1)) {
						if ((slave.attrXX > 85) && (V.PC.boobs >= 400)) {
							r.push(`${His} attraction to women is obvious: ${he} can't seem to stop staring at your breasts.`);
						} else if ((slave.attrXY > 85) && (V.PC.dick !== 0)) {
							r.push(`${His} attraction to men is obvious: ${he} can't seem to stop glancing down at your package.`);
						} else if ((slave.attrXY > 85) && V.PC.title === 1 && (V.PC.boobs < 300)) {
							r.push(`${His} attraction to men is obvious: ${he} can't seem to stop checking out your broad chest.`);
						}
					}
				}
			}
		}
	}

	r.push(App.Desc.family(slave));

	r.push(App.Desc.relationRival(slave));

	if (slave.bodySwap > 0) {
		if (slave.origBodyOwner !== "") {
			r.push(`${He} currently possesses ${slave.origBodyOwner}'s body.`);
		}
		if (slave.fetish !== "mindbroken" && slave.fuckdoll === 0 && slave.origBodyOwnerID > 0) {
			let lsd = V.slaveIndices[slave.origBodyOwnerID];
			if (lsd) {
				r.push(`${He} is fully aware that ${SlaveFullName(V.slaves[lsd])} is in ${his} old body.`);
			}
		}
	}
	$(p).append(r.join(` `));
	el.appendChild(p);

	p = document.createElement("p");
	p.className = "indent";
	r = [];
	if (!market) {
		if (slave.origin !== 0) {
			r.push(pronounsForSlaveProp(slave, slave.origin));
		}
	}

	r.push(App.Desc.career(slave));

	r.push(App.Desc.prestige(slave));

	r.push(App.Desc.skills(slave));

	if (slave.custom.desc !== "" && slave.custom.desc) {
		r.push(pronounsForSlaveProp(slave, slave.custom.desc));
	}

	if (V.arcologies[0].FSGenderFundamentalistLawBeauty + V.arcologies[0].FSGenderRadicalistLawBeauty > 0) {
		if (genderLawPass(slave) === 1) {
			r.push(`${His} body is a perfect example of the fashionable feminine ideal.`);
		}
	} else if (V.arcologies[0].FSSlimnessEnthusiastLaw === 1) {
		if (slimLawPass(slave) === 1) {
			r.push(`${His} body is a perfect example of the fashionable ideal flat body.`);
		}
	} else if (V.arcologies[0].FSHedonisticDecadenceLaw2 === 1) {
		if (slave.boobs >= 2000 && slave.butt >= 5 && slave.weight > 95) {
			r.push(`${His} body is a perfect example of the fashionable plump body.`);
		}
	}

	if (V.arcologies[0].FSRestartSMR > 0 && V.arcologies[0].FSRestart > 80 && (slave.dick === 0) && (slave.balls === 0) && (slave.ovaries === 0) && (slave.mpreg === 0) && (slave.vagina < 0)) {
		r.push(`${His} body is a perfect example of a subhuman who has accepted their place in society.`);
	}

	if (V.arcologies[0].FSIntellectualDependencyLawBeauty === 1) {
		r.push(`${He} currently is graded`);
		switch (bimboScore(slave)) {
			case 6:
				r.push(`an S-class`);
				break;
			case 5:
				r.push(`an A-class`);
				break;
			case 4:
				r.push(`a B-class`);
				break;
			case 3:
				r.push(`a C-class`);
				break;
			case 2:
				r.push(`a D-class`);
				break;
			case 1:
				r.push(`an E-class`);
				break;
			default:
				r.push(`an F-class`);
		}
		r.push(`bimbo by societal trends.`);
	}

	$(p).append(r.join(" "));

	if (V.showScores !== 0) {
		p.append(` Currently, ${he} has an `);

		// Beauty
		App.UI.DOM.appendNewElement("span", p, `attractiveness score `, ["pink", "bold"]);
		App.UI.DOM.appendNewElement("span", p, `of `, ["pink"]);
		p.append(BeautyTooltip(slave), ` and a `);

		// Fresult
		App.UI.DOM.appendNewElement("span", p, `sexual score `, ["lightcoral", "bold"]);
		App.UI.DOM.appendNewElement("span", p, `of `, ["lightcoral"]);
		p.append(FResultTooltip(slave));
	}

	el.appendChild(p);
	p = document.createElement('p');
	p.className = "indent";
	r = [];

	r.push(App.Desc.limbs(slave));
	r.push(App.Desc.clothing(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.armwear(slave));

	if (V.showBodyMods === 1) {
		r.push(App.Desc.clothingCorset(slave));
	}

	if (V.showClothing === 1 && !market) {
		r.push(`${He}`);
	} else {
		r.push(`${slave.slaveName}`);
	}

	r.push(App.Desc.dimensions(slave));
	r.push(App.Desc.bodyguard(slave));

	if ((slave.counter.pitWins + slave.counter.pitLosses) > 0) {
		r.push(`${He} has participated in ${num(slave.counter.pitWins + slave.counter.pitLosses)} pit fights, with ${slave.counter.pitWins} wins and ${slave.counter.pitLosses} losses.`);
	}

	if (slave.counter.pitKills > 0) {
		r.push(`${slave.counter.pitKills} slaves have died by ${his} hand in pit fights.`);
	}

	r.push(App.Desc.piercing(slave, "corset"));
	r.push(App.Desc.pregnancy(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.legs(slave));
	r.push(App.Desc.mods(slave, "thigh"));
	r.push(App.Desc.mods(slave, "calf"));
	r.push(App.Desc.mods(slave, "ankle"));
	r.push(App.Desc.mods(slave, "foot"));
	r.push(App.Desc.heels(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.skin(slave, {
		market: market, eventDescription: eventDescription
	}));

	if (market) {
		r.push(App.Desc.accent(slave));
	}

	let scarCounter = 0;
	for (let scarName in slave.scar) {
		if (slave.ID === V.BodyguardID && scarCounter > 1) {
			r.push(`${His} scars make ${him} look even more menacing than ${he} actually is.`);
			break;
		} else if ((slave.ID === V.BodyguardID) && slave.scar[scarName].menacing > 0) {
			r.push(`${His} menacing scar makes ${him} look terrifying.`);
			break;
		} else if ((slave.ID === V.WardenessID) && scarCounter > 1) {
			r.push(`${His} scars make ${him} look like ${he}'s in the right place.`);
			break;
		}
		scarCounter++;
	}

	if (slave.fuckdoll === 0) {
		if (slave.markings === "birthmark" && slave.prestige === 0 && slave.porn.prestige < 2) {
			r.push(`${He} has a large, liver-colored birthmark, detracting from ${his} beauty.`);
		}
		if (slave.skin === "sun tanned") {
			if ((slave.rules.release.slaves === 1) || App.Utils.hasFamilySex(slave)) {
				if (slave.fetishStrength > 60 && slave.fetishKnown === 1) {
					r.push(`${His} tan is slightly uneven, since ${he} enjoys`);
					if (slave.fetish === "buttslut") {
						r.push(`letting other tanned slaves share a tanning bed with ${him} so they can sodomize ${him} while ${he} tans.`);
					} else if ((slave.fetish === "cumslut")) {
						r.push(`letting other tanned slaves share a tanning bed with ${him} so they get oral from ${him} while ${he} tans.`);
					} else if ((slave.fetish === "sadist")) {
						r.push(`forcing inferior slaves into the tanning beds with ${him} so ${he} can sodomize them while ${he} tans.`);
					} else if ((slave.fetish === "dom")) {
						r.push(`bringing other slaves into the tanning beds with ${him} so ${he} can fuck them while ${he} tans.`);
					} else if ((slave.fetish === "masochist") || (slave.fetish === "submissive")) {
						r.push(`letting other slaves into the tanning beds with ${him} so they can fuck ${him} while ${he} tans.`);
					} else if ((slave.fetish === "boobs")) {
						r.push(`bringing other slaves into the tanning beds with ${him} so ${he} can tittyfuck them while ${he} tans.`);
					} else if ((slave.fetish === "pregnancy") && (jsRandom(0, 99) < V.seeDicks)) {
						r.push(`letting slaves with dicks into the tanning beds with ${him} so they can cum inside ${him} while ${he} tans.`);
					} else {
						r.push(`bringing other slaves into the tanning beds with ${him} to have sex while ${he} tans.`);
					}
				}
			}
		}
	}

	if (slave.fuckdoll === 0) {
		// Describe any brands that are not directly addressed elsewhere in longSlave
		r.push(App.Desc.mods(slave, "extra"));
	}

	if (V.showClothing === 1 && !market) {
		r.push(App.Desc.ears(slave));
		r.push(App.Desc.upperFace(slave));
		r.push(App.Desc.hair(slave));
	} else {
		r.push(`${His} hair is`);
		if (slave.hColor !== slave.eyebrowHColor) {
			r.push(`${slave.hColor}, with ${slave.eyebrowHColor} eyebrows.`);
		} else {
			r.push(`${slave.hColor}.`);
		}
	}

	if (slave.fuckdoll === 0) {
		if (slave.hColor === "red") {
			if (slave.hLength >= 10) {
				if (slave.markings === "freckles" || slave.markings === "heavily freckled") {
					if (skinToneLevel(slave.skin) > 5 && skinToneLevel(slave.skin) < 10) {
						r.push(`It goes perfectly with ${his} ${slave.skin} skin and freckles.`);
					}
				}
			}
		}

		const pubertyAge = Math.min(slave.pubertyAgeXX, slave.pubertyAgeXY);
		if (slave.physicalAge < pubertyAge - 2) {
			r.push(`${He} is too sexually immature to have armpit hair.`);
		} else if (slave.underArmHStyle === "hairless") {
			r.push(`${His} armpits are perfectly smooth and naturally hairless.`);
		} else if (slave.underArmHStyle === "bald") {
			r.push(`${His} armpits no longer grow hair, leaving them smooth and hairless.`);
		} else if (slave.underArmHStyle === "waxed") {
			if (slave.assignment === "work in the dairy" && V.dairyRestraintsSetting > 1) {
				r.push(`${His} armpit hair has been removed to prevent chafing.`);
			} else {
				r.push(`${His} armpits are waxed and smooth.`);
			}
		} else if (slave.physicalAge < pubertyAge - 1) {
			r.push(`${He} has a few ${slave.underArmHColor} wisps of armpit hair.`);
		} else if (slave.physicalAge < pubertyAge) {
			r.push(`${He} is on the verge of puberty and has a small patch of ${slave.underArmHColor} armpit hair.`);
		} else if (slave.underArmHStyle === "shaved") {
			r.push(`${His} armpits appear hairless, but closer inspection reveals light, ${slave.underArmHColor} stubble.`);
		} else if (slave.underArmHStyle === "neat") {
			r.push(`${His} armpit hair is neatly trimmed`);
			if (!hasBothArms(slave)) {
				r.push(`since`);
				if (hasAnyArms(slave)) {
					r.push(`at least half`);
				} else {
					r.push(`it`);
				}
				r.push(`is always in full view.`);
			} else {
				r.push(`to not be visible unless ${he} lifts ${his} arms.`);
			}
		} else if (slave.underArmHStyle === "bushy") {
			r.push(`${His} ${slave.underArmHColor} armpit hair has been allowed to grow freely,`);
			if (!hasAnyArms(slave)) {
				r.push(`creating two bushy patches under where ${his} arms used to be.`);
			} else {
				r.push(`so it can be seen poking out from under ${his}`);
				if (hasBothArms(slave)) {
					r.push(`arms`);
				} else {
					r.push(`arm`);
				}
				r.push(`at all times.`);
			}
		}
	}

	$(p).append(r.join(` `));
	p.append(` `);

	if (slave.voice === 0) {
		p.append(`${He} is `);
		span = document.createElement('span');
		span.className = "pink";
		span.textContent = `completely silent, `;
		p.appendChild(span);
		p.append(` which is understandable, since ${he}'s mute. `);
	} else if (slave.lips > 95) {
		p.append(`${He} is `);
		span = document.createElement('span');
		span.className = "pink";
		span.textContent = `effectively mute, `;
		p.appendChild(span);
		p.append(` since ${his} lips are so large that ${he} can no longer speak intelligibly. ${He} can still `);
		if (slave.devotion > 50) {
			p.append(`moan `);
		} else if (slave.devotion > 20) {
			p.append(`whimper `);
		} else {
			p.append(`scream `);
		}
		p.append(`through them, though. `);
	}

	r = [];

	if (V.showBodyMods === 1) {
		if (slave.fuckdoll > 0) {
			if (slave.earPiercing + slave.eyebrowPiercing + slave.nosePiercing > 0) {
				r.push(`The piercings on ${his} head run through ${his} suit, helping secure the material to ${his} head.`);
			}
		} else {
			r.push(App.Desc.mods(slave, "ear"));
			r.push(App.Desc.mods(slave, "nose"));
			r.push(App.Desc.mods(slave, "eyebrow"));
			r.push(App.Desc.mods(slave, "cheek"));
			r.push(App.Desc.mods(slave, "neck"));
			if (slave.custom.tattoo !== "" && slave.custom.tattoo) {
				r.push(pronounsForSlaveProp(slave, slave.custom.tattoo));
			}
		}
	}

	r.push(App.Desc.horns(slave));
	r.push(App.Desc.face(slave));
	r.push(App.Desc.mouth(slave));

	if (V.showClothing === 1 && !market) {
		if (slave.fuckdoll === 0) {
			r.push(App.Desc.collar(slave));
			r.push(App.Desc.faceAccessory(slave));
			r.push(App.Desc.mouthAccessory(slave));
			if (slave.relationship > 4) {
				if (hasAnyArms(slave)) {
					r.push(`${He} has a simple gold band on the little finger of ${his}`);
					if (!hasLeftArm(slave)) {
						r.push(`right`);
					} else {
						r.push(`left`);
					}
					r.push(`hand.`);
				} else {
					r.push(`${He} has a simple gold band on a length of chain around ${his} neck.`);
				}
			} else if (slave.relationship === -3) {
				if (hasAnyArms(slave)) {
					r.push(`${He} has a simple steel band on the little finger of ${his}`);
					if (!hasLeftArm(slave)) {
						r.push(`right`);
					} else {
						r.push(`left`);
					}
					r.push(`hand.`);
				} else {
					r.push(`${He} has a simple steel band on a length of cord around ${his} neck.`);
				}
			}
		}
	}

	if (slave.fuckdoll === 0) {
		r.push(App.Desc.nails(slave));
	}
	r.push(App.Desc.mods(slave, "back"));
	r.push(App.Desc.mods(slave, "shoulder"));
	r.push(App.Desc.mods(slave, "upper arm"));
	r.push(App.Desc.mods(slave, "lower arm"));
	r.push(App.Desc.mods(slave, "hand"));
	r.push(App.Desc.mods(slave, "wrist"));

	$(p).append(r.join(` `));
	p.append(` `);

	if (slave.fuckdoll === 0) {
		if (slave.minorInjury !== 0) {
			if (slave.minorInjury !== "sore ass") {
				p.append(`${He} is sporting a `);
				span = document.createElement('span');
				span.className = "red";
				span.textContent = `${slave.minorInjury}, `;
				p.appendChild(span);
				p.append(` covered by makeup. `);
			}
		}
	}
	if (slave.health.illness > 0) {
		if (slave.fuckdoll === 0) {
			p.append(`${He} `);
		} else {
			p.append(`${His} suit reports that ${he} `);
		}
		span = document.createElement('span');
		if (slave.health.illness === 1) {
			if (slave.fuckdoll === 0) {
				p.append(`is `);
				span.className = "red";
				span.textContent = `feeling under the weather. `;
				p.appendChild(span);
			} else {
				p.append(`has `);
				span.className = "red";
				span.textContent = `fallen ill. `;
				p.appendChild(span);
			}
		} else if (slave.health.illness === 2) {
			p.append(`is `);
			span.className = "red";
			span.textContent = `somewhat ill. `;
			p.appendChild(span);
		} else if (slave.health.illness === 3) {
			p.append(`is `);
			span.className = "red";
			span.textContent = `sick. `;
			p.appendChild(span);
		} else if (slave.health.illness === 4) {
			p.append(`is `);
			span.className = "red";
			span.textContent = `very sick. `;
			p.appendChild(span);
		} else if (slave.health.illness === 5) {
			p.append(`is `);
			span.className = "red";
			span.textContent = `terribly ill. `;
			p.appendChild(span);
		}
	}

	el.appendChild(p);
	p = document.createElement("p");
	p.className = "indent";
	r = [];
	// Calling all boob widgets
	r.push(App.Desc.boobs(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.boobsShape(slave));
	r.push(App.Desc.boobsExtra(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.mods(slave, "chest"));
	r.push(App.Desc.mods(slave, "breast"));
	r.push(App.Desc.shoulders(slave));
	r.push(App.Desc.nipples(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.mods(slave, "nipple"));
	r.push(App.Desc.areola(slave, {
		market: market, eventDescription: eventDescription, applyLaw: applyLaw
	}));
	r.push(App.Desc.mods(slave, "areolae"));

	if (slave.inflation > 0) {
		// to be obsoleted with phase 4
		r.push(App.Desc.bellyInflation(slave, {
			market: market, eventDescription: eventDescription
		}));
	} else if (slave.bellyImplant >= 2000) {
		r.push(App.Desc.bellyImplant(slave, {
			market: market, eventDescription: eventDescription
		}));
	} else {
		r.push(App.Desc.belly(slave, {
			market: market, eventDescription: eventDescription
		}));
	}
	r.push(App.Desc.mods(slave, "belly"));
	r.push(App.Desc.butt(slave, {
		market: market, eventDescription: eventDescription
	}));

	$(p).append(r.join(` `));
	el.appendChild(p);

	p = document.createElement("p");
	p.className = "indent";
	r = [];

	r.push(App.Desc.crotch(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.dick(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.vagina(slave, {
		market: market, eventDescription: eventDescription
	}));
	r.push(App.Desc.anus(slave, {
		market: market, eventDescription: eventDescription
	}));

	$(p).append(r.join(` `));
	el.appendChild(p);

	if (slave.fuckdoll === 0) {
		p = document.createElement("p");
		p.className = "indent";
		$(p).append(App.Desc.drugs(slave));
		el.appendChild(p);
	}

	// clear sale and law flags, if set

	return el;
};
