/*
receiver - What is the state of the network box.
	-1: PC has not been given a welcome packet. (default)
	0: PC has been given a welcome packet but has not installed it.
	1+: box is installed.
channel[numberAsString] - how many times it has been viewed.
channel.last - program viewed last week.
pcViewership.count - How many weeks since the PC last watched FCTV.
pcViewership.frequency - How often should the PC watch FCTV.
	-1: Never.
	1: at least once a week.
	2: least every two weeks.
	4: at least once a month. (default)
remote - Does the PC have a FCTV branded remote (also used to trigger a slave acquisition event).
weekEnabled - The week FCTV was installed.
*/
globalThis.FCTV = (function() {
	return {
		channels: channels,
		initChannels: initChannels,
		channelCount: channelCount,
		FinalTouches: FinalTouches,
	};

	/** Produce an array of numbers representing all available channels in App.Data.FCTV.channels
	 * @returns {number[]}
	 */
	function channels() {
		return Object.keys(App.Data.FCTV.channels).map(chStr => parseInt(chStr, 10));
	}

	function initChannels() {
		const channelList = FCTV.channels();
		for (let i = 0; i < channelList.length; i++) {
			V.FCTV.channel[num(channelList[i], true)] = 0;
		}
	}

	/**
	 * checks value of selected channel in relation to an arbitrary number.  Have we watched it more/less/same than i.
	 * @param {number} selectedChannel
	 * @param {number} i
	 * @param {string} operation eq/gt/lt: equals/greater than/less than.
	 * @returns {boolean}
	 */
	function channelCount(selectedChannel, i, operation = 'eq') {
		if (operation === 'eq') {
			if (V.FCTV.channel[num(selectedChannel, true)] === i) {
				return true;
			}
		} else if (operation === 'gt') {
			if (V.FCTV.channel[num(selectedChannel, true)] > i) {
				return true;
			}
		} else if (operation === 'lt') {
			if (V.FCTV.channel[num(selectedChannel, true)] < i) {
				return true;
			}
		}
		return false;
	}
	/**
	 * Applies some universal changes to FCTV slaves
	 * @param {App.Entity.SlaveState} slave
	 * @returns {App.Entity.SlaveState}
	 */
	function FinalTouches(slave) {
		slave.pubertyXX = 1;
		slave.career = "a slave";
		slave.origin = "You purchased $him from FCTV's Home Slave Shopping stream channel.";
		setHealth(slave, 80, 0, 0, 0, 0);
		return slave;
	}
})();

globalThis.FctvDisplay = function({usedRemote = 0, seeAll = 0, selectedChannel = 0} = {}) {
	return watch({usedRemote: usedRemote, seeAll: seeAll, selectedChannel: selectedChannel});

	function watch({usedRemote = 0, seeAll = 0, selectedChannel = 0} = {}) {
		const frag = new DocumentFragment();
		let p;
		let div;
		let span;
		V.FCTV.pcViewership.count++;
		let _possibleChannels = FCTV.channels();

		p = document.createElement("p");

		p.append(`Tired after a long day, you tell `);
		if (S.Concubine) {
			p.append(`your concubine: `);
			App.UI.DOM.appendNewElement("span", p, S.Concubine.slaveName, "pink");
		} else {
			p.append(V.assistant.name);
		}
		p.append(` to turn on the TV and `);
		if (!usedRemote) {
			p.append(`set FCTV to find a random show. Your larger-than-life screen flashes on, and is soon playing content from the popular streaming service. `);
			if (V.cheatMode > 0 || V.debugMode > 0 || V.FCTV.remote) {
				// Create "Use remote" link.  Hide once clicked.
				span = document.createElement("span");
				span.id = "use-remote";
				span.append(
					App.UI.DOM.link(
						"Use your remote",
						() => {
							usedRemote = 1;
							jQuery("#remote").empty().append(createRemote());
							jQuery("#use-remote").empty();
						}
					)
				);
				p.append(span);
			}
			p.append(randomShow());
		} else if (usedRemote && (V.cheatMode > 0 || V.debugMode > 0 || V.FCTV.remote)) {
			p.append(`bring you the remote so you can choose whatever show you find interesting. `);
			if (selectedChannel === V.FCTV.channel.last) {
				App.UI.DOM.appendNewElement("div", p, `You tuned into this channel last week, you may want to choose something else.`);
			}
			if (seeAll) {
				div = document.createElement("div");
				div.append(`There is an audible tone from your screen, which then displays a message: `);
				App.UI.DOM.appendNewElement("span", div, `Showing all content, for testing purposes.`, "note");
				p.append(div);
			}
		}

		frag.append(p);

		p = document.createElement("p");
		p.classList.add("FCTV");
		span = document.createElement("span");
		span.id = "remote";
		if (usedRemote && (V.cheatMode > 0 || V.debugMode > 0 || V.FCTV.remote)) {
			span.append(createRemote());
		}
		p.append(span);
		if (usedRemote) {
			p.append(`You select `);
		} else {
			p.append(`It looks like the random function chose `);
		}
		App.UI.DOM.appendNewElement("span", p, `channel number: ${selectedChannel}`, "bold");

		// A little glue to get unique data based stuff to stick.
		if (selectedChannel === 16) {
			p.append(`. `);
		} else {
			p.append(`, `);
			if (selectedChannel === 11 && usedRemote) {
				p.append(`for some foolish reason`);
			}
		}
		p.append(displayShow());
		frag.append(p);
		return jQuery("#fctv-watch").empty().append(frag);

		/** Creates a remote control for FCTV
		 * @returns {HTMLParagraphElement}
		 */
		function createRemote() {
			const p = document.createElement("p");
			p.classList.add("fctv-remote");
			let div;
			let span;
			const buttons = [];

			for (const i of _possibleChannels) {
				if (showChannel(i).canSelect > 0) {
					if (selectedChannel !== i) { // Selected button
						buttons.push(
							App.UI.DOM.link(
								i.toFixed(0),
								() => {
									watch({usedRemote: 1, seeAll: seeAll, selectedChannel: i});
								}
							)
						);
					} else { // Channel you can choose
						span = document.createElement("span");
						span.classList.add("white");
						span.style.fontWeight = "bold";
						span.append(
							App.UI.DOM.link(
								i.toFixed(0),
								() => {
									watch({usedRemote: 1, seeAll: seeAll, selectedChannel: i});
								},
								[],
								"",
								`Current channel`
							)
						);
						buttons.push(span);
					}
				} else { // Channel you cannot choose
					buttons.push(
						App.UI.DOM.disabledLink(i.toFixed(0), [showChannel(i).text])
					);
				}
			}

			buttons.splice(4, 0, App.UI.DOM.makeElement("span", "4")); // 4 is no longer a real channel, so put in a dummy button. 404, channel not found.

			div = document.createElement("div");
			div.classList.add("fctv-remote-numbers", /* "fctv-remote-button" */);
			for (const button of buttons) {
				div.append(button);
			}
			p.append(div);
			App.UI.DOM.appendNewElement(
				"div",
				p,
				App.UI.DOM.link(
					`Random`,
					() => {
						watch({usedRemote: 0, seeAll: seeAll, selectedChannel: selectedChannel});
					}
				),
				["fctv-remote-button"]
			);
			if (V.cheatMode > 0 || V.debugMode > 0) {
				App.UI.DOM.appendNewElement(
					"div",
					p,
					App.UI.DOM.link(
						`Toggle inappropriate`,
						() => {
							if (seeAll) {
								seeAll = 0;
							} else {
								seeAll = 1;
							}
							watch({usedRemote: usedRemote, seeAll: seeAll, selectedChannel: selectedChannel});
						}
					),
					["fctv-remote-button"]
				);
			}
			return p;
		}

		function randomShow() {
			const frag = new DocumentFragment();

			if (!usedRemote) {
				const channels = FCTV.channels();
				channels.push(3); // Double chance for slave sale. Replaces old channel 4.
				for (let i = 0; ; i++){
					// Roll for a channel
					const channel = _.sample(channels);
					// See if we can show it
					const x = showChannel(channel);
					if (x.canSelect === -1) {
						App.UI.DOM.appendNewElement("p", frag, channelFailed(x.text));
					} else {
						selectedChannel = channel;
						break;
					}

					// Three strikes, then move on
					if (i > 2) {
						App.UI.DOM.appendNewElement("p", frag, `It looks like there is simply nothing on FCTV tonight worth watching.`);
						return frag;
					}
				}
			}
			return frag;
		}

		function showChannel(i = 1) {
			let x = {canSelect: 1};
			if (seeAll) {
				return x;
			}
			if (App.Data.FCTV.channels.hasOwnProperty(i)) {
				if (App.Data.FCTV.channels[i].hasOwnProperty("tags")) {
					x = checkTags(App.Data.FCTV.channels[i].tags);
				}
				if (App.Data.FCTV.channels[i].hasOwnProperty("disableSelection")) {
					if (usedRemote !== 0) {
						x.canSelect = -1;
					}
					x.text = `This channel appears at random times`;
				}
			} else {
				throw `Channel "${i}" does not exist`;
			}
			if (i === 11) {
				if (V.purchasedSagBGone && V.FCTV.channel[num(i, true)] > 2) {
					x.canSelect = -1; x.text = `Product purchase detected, skipping commercials`;
				}
			}
			return x;
		}

		/**
		 * Text to frame if content won't work due to tags.
		 * @param {string} text Reason this channel/episode can't be shown
		 * @returns {Node}
		 */
		function channelFailed(text) {
			const frag = new DocumentFragment;
			frag.append(`A notification is shown: `);
			App.UI.DOM.appendNewElement("span", frag, text, "note");
			frag.append(`, changing program.`);
			return frag;
		}

		/**
		 * Displays just the channel itself, including art
		 * @returns {Node}
		 */
		function displayShow() {
			const frag = new DocumentFragment();

			/** @type {FctvChannel} */
			const channel = App.Data.FCTV.channels[selectedChannel];
			const epToShow = getEpisode(selectedChannel);

			// Fail code, so we fail
			if (epToShow === -1) {
				frag.append(`no valid episodes`);
				return frag;
			}

			// Increment the viewing record for this channel
			V.FCTV.channel[num(selectedChannel, true)]++;

			// Slave, if needed.  Hosts and market slaves.
			let slave;
			if (channel.episode[epToShow].slaves) {
				slave = channel.episode[epToShow].slaves[0];
				if (!channel.disableSelection) { // Art for the slave market will be shown in longSlave
					App.Events.drawEventArt(frag, channel.episode[epToShow].slaves);
				}
			}
			if (channel.intro) {
				$(frag).append(channel.intro);
			}
			const episodeText = channel.episode[epToShow].text;
			if (episodeText) {
				if (typeof episodeText === 'function') {
					$(frag).wiki(episodeText(slave));
				} else {
					$(frag).wiki(episodeText);
				}
			}
			if (channel.outro) {
				if (typeof channel.outro === 'function') {
					frag.append(channel.outro(slave, epToShow));
				} else {
					frag.append(channel.outro);
				}
			}
			V.FCTV.channel.last = selectedChannel;
			return frag;

			function getEpisode(sel) {
				let epToShow = -1; // -1 is the fail code.
				const epsArray = [];
				/** @type {FctvChannel} */
				const channel = App.Data.FCTV.channels[sel];
				const viewedCount = V.FCTV.channel[num(sel, true)] || 0;

				// Produce an array of episodes we can watch.
				for (let i = 0; i < App.Data.FCTV.channels[sel].episode.length; i++) {
					const ep = App.Data.FCTV.channels[sel].episode[i];
					if (ep.tags) {
						const x = checkTags(ep.tags);
						if (x.canSelect !== -1) {
							epsArray.push(i);
						}
					} else {
						epsArray.push(i);
					}
				}
				const availableEp = epsArray.length;
				if (epsArray.length === 0) {
					return -1; // Nothing to watch, fail.
				}

				if (availableEp > viewedCount) { // If we watched ep 0 last time, our view count will be 1.  Now we can use 1 as our new ep, etc.
					epToShow = viewedCount;
				} else if (channel.loop === true) {
					// How many times have we been through this series.  Lets say we watched 10 episodes, but there are only 3 uniques [0,1,2].
					const watchedEntireSeason = Math.trunc(viewedCount / availableEp); // we went through 3 times fully
					epToShow = viewedCount - (watchedEntireSeason * availableEp); // 10 - 3 seasons (9) is 1. So our last episode was the first, 0 in the array.  And 1 is the next ep!
				} else { // We have seen all the episodes, return a random one
					epToShow = jsRandom(0, availableEp-1);
				}
				if (epToShow === -1) {
					return epToShow;
				}
				return epsArray[epToShow];
			}
		}

		/**
		 * Checks the tags on a channel or an episode to determine if it can be shown.
		 * @param {FctvTags} tags
		 * @returns {Object} x
		 */
		function checkTags(tags){
			let x = {
				canSelect: 1,
			};
			for (const tag in tags) {
				if (tag) {
					switch (tag) {
						case "hyperPreg":
							if (!V.seeHyperPreg) {
								x.canSelect = -1; x.text = `Too much happiness detected`;
							}
							break;
						case "preg":
							if (!V.seePreg) {
								x.canSelect = -1; x.text = `Too much baking detected`;
							}
							break;
						case "extreme":
							if (!V.seeExtreme) {
								x.canSelect = -1; x.text = `Too much hugging detected`;
							}
							break;
						case "dicks":
							if (!V.seeDicks && !V.makeDicks) {
								x.canSelect = -1; x.text = `Too many hot dogs detected`;
							}
							break;
						case "incest":
							if (!V.seeIncest && tag === "incest") {
								x.canSelect = -1; x.text = `Too much familiarity detected`;
							}
							break;
						case "loli":
							if (V.minimumSlaveAge > 13 && tag === "loli") {
								x.canSelect = -1; x.text = `Actor not vintage enough`;
							}
							break;
						default:
							throw `Tag "${tag}" unknown for ${tags}`;
					}
				}
			}
			return x;
		}
	}
};
